
from rest_framework.exceptions import ValidationError
from wagtail_rest_pack.recaptcha.models import RecaptchaVerifier
from django.conf import settings
class NoopRecaptchaVerifier(RecaptchaVerifier):
    def verify(self, request):
        pass


from wagtail_rest_pack.generic_forms.blocks.generic_input import GenericInputBlock
from django.core.exceptions import ValidationError
from django.db import models
from modelcluster.models import ClusterableModel
from wagtail.snippets.models import register_snippet
from django.db import models
from rest_framework import serializers
from wagtail.core import blocks

from wagtail.core.fields import StreamField


class InputBlockSerializer(serializers.Serializer):

    block_name = 'form_text'

    @staticmethod
    def block_definition():
        return InputBlockSerializer.block_name, GenericInputBlock(specific=[
            ('multiline', blocks.BooleanBlock(required=False, help_text="Víceřádkový vstup."))
        ])

    def to_representation(self, instance):
        return instance


from django.db import models
from modelcluster.fields import ParentalKey
from wagtail.core.models import Orderable, Page
from wagtail.snippets.edit_handlers import SnippetChooserPanel

from modelcluster.models import ClusterableModel
from wagtail_rest_pack.sitesettings.snippets import ImageSliderItem


class SiteSettingsSection(Orderable, models.Model):
    page = ParentalKey(Page, on_delete=models.CASCADE, related_name='carouselitems')
    slider = models.ForeignKey(ImageSliderItem, on_delete=models.CASCADE, related_name='+')
    panels = [
        SnippetChooserPanel('slider'),
    ]

    class Meta:
        verbose_name = "Carousel Item"
        verbose_name_plural = "Carousel Items"



from .response import FormResponse
from wagtail.core import blocks

class ShowDialogResponse(FormResponse):
    type= 'form_open_dialog'

    @staticmethod
    def block_definition() ->tuple:
        return ShowDialogResponse.type, blocks.StructBlock(local_blocks=[
            ('title', blocks.TextBlock(required=True, help_text="Titulek", max_length=40)),
            ('text', blocks.StreamBlock([
                ('richtext', blocks.RichTextBlock(icon="doc-full"))
            ]))
        ])



from rest_framework.permissions import BasePermission

from wagtail_rest_pack.comments.models import Comment


class CommentOwnerOrStaff(BasePermission):

    def has_object_permission(self, request, view, obj: Comment):
        user =request.user
        if user  is None:
            return False
        created_by = obj.created_by
        is_staff = user.is_staff or user.is_superuser
        if created_by is not None and created_by.id == user.id:
            return True
        return is_staff





import re

from django.core.exceptions import ValidationError
from wagtail.core import blocks

from rest_framework import serializers
def is_form_field(value):
    if not isinstance(value, str):
        raise ValidationError("Only string is allowed.")
    pattern = "^[a-zA-Z0-9_-]+$"
    pass
    if re.match(pattern, value) is None:
        raise ValidationError("Does not match a-z, A-Z")


class GenericInputBlock(blocks.StructBlock):
    def __init__(self, specific: [] = None, **kwargs):
        # todo validace na unikátnost name
        local_blocks = [
            ('name', blocks.TextBlock(max_length=30, help_text='Název pole v ascii (např. user_email)',
                                      label='Technický název pole', validators=[is_form_field])),
            ('label', blocks.TextBlock(max_length=50, label='Popisek zobrazený před inputem.')),
            ('max_length',
             blocks.IntegerBlock(label='Max', help_text='Maximální délka vstupu', default=100, min_value=10,
                                 max_value=2000)),
            ('required', blocks.ChoiceBlock(choices=[
                ('always', 'Vždy'),
                ('anonymous_user_only', 'Pouze nepřihlášený uživatel')
            ], default='always', label='Podmínka zobrazení pole')),
            ('placeholder',
             blocks.CharBlock(max_length=80, required=False, help_text='Hodnota zobrazená, když nic není vyplněno.')),
            ('validation', blocks.ChoiceBlock(choices=[
                ('none', 'Žádné'),
                ('email', 'Email')
            ], default='none', label='Validace'))
        ]
        if specific is not None:
            local_blocks.extend(specific)
        super().__init__(local_blocks=local_blocks, **kwargs)


# def validator(request, value, validated_field, value_field, field, add_error):
#     # Required
#     if value and value_field['required'] == 'anonymous_user_only' and request.user.is_authenticated:
#         add_error(validated_field, 'Pole je povinné pouze pro nepřihlášené uživatele')
#         return
#     if not value:
#         if value_field['required'] == 'anonymous_user_only' and not request.user.is_authenticated:
#             add_error(validated_field, 'Toto pole je požadováno.')
#         return
#
#     # Type
#     if field['type'] == get_form_base_name() + 'text':
#         if not isinstance(value, str):
#             add_error(validated_field, 'Hodnota není typu str')
#     else:
#         add_error(validated_field, 'Neočekávaný typ ' + field['type'])
#
#     # Max Length
#     if len(value) > value_field['max_length']:
#         # todo template string
#         add_error(validated_field, 'Délka hodnoty je moc velká. Očekáváno max ' + str(
#             value_field['max_length']) + ', ale je ' + str(len(value)))
#
#     # Custom Validation
#     if value_field['validation'] == 'email':
#         try:
#             validate_email(value)
#         except ValidationError as err:
#             add_error(validated_field, 'Toto není validní email.')



from rest_framework import serializers
from wagtail.core import blocks
from wagtail_rest_pack.generic_forms.actions.send_email import SendEmailAction
from wagtail_rest_pack.generic_forms.response.snack import ShowSnackResponse
from wagtail_rest_pack.generic_forms.response.dialog import ShowDialogResponse


class SubmitBlockSerializer(serializers.Serializer):
    block_name = 'form_submit'
    name = serializers.CharField(max_length=30)
    text = serializers.CharField(max_length=30)

    @staticmethod
    def block_definition():
        return SubmitBlockSerializer.block_name, SubmitBlock('submit')

    class Meta:
        fields = ('name', 'text',)




class SubmitBlock(blocks.StructBlock):

    def get_responses(self):
        return [
            ShowDialogResponse.block_definition(),
            ShowSnackResponse.block_definition(),
        ]

    def get_actions(self):
        return [
            SendEmailAction.block_definition(),
        ]

    def __init__(self, *args, **kwargs):
        self.action_definitions = blocks.StreamBlock(self.get_actions(), min_num=1, max_num=1, label="Akce")
        self.response_definition = blocks.StreamBlock(self.get_responses(), min_num=1, max_num=1, label="Odpověď")
        super().__init__(local_blocks=[
            ('name', blocks.TextBlock(max_length=30, help_text="Název pole v ascii (např. user_email)", label="Jméno", validators=[])),
            ('text', blocks.CharBlock(max_length=30, label="Text na tlačítku")),
            ('action', self.action_definitions),
            ('response', self.response_definition)
        ], **kwargs)



from wagtail.core import blocks
from wagtail_rest_pack.generic_forms.blocks.generic_input import GenericInputBlock
from django.core.exceptions import ValidationError
from django.db import models
from modelcluster.models import ClusterableModel
from wagtail.snippets.models import register_snippet
from django.db import models
from rest_framework import serializers
from wagtail.core import blocks

from wagtail.core.fields import StreamField
from wagtail_rest_pack.generic_forms.blocks.text_block import InputBlockSerializer
from wagtail_rest_pack.streamfield.serializers import StreamFieldSerializer, SettingsStreamFieldSerializer
choices = [
    ('always', "Vždy"),
    ('anonymous_user_only', "Pouze nepřihlášený uživatel")
]

class GroupBlockSerializer(serializers.Serializer):
    block_name = 'form_group'
    name = serializers.CharField(max_length=80)
    row = serializers.BooleanField()
    required = serializers.ChoiceField(choices=choices)
    stream = SettingsStreamFieldSerializer()
    class Meta:
        fields = ('name', 'row', 'required', 'stream',)

    @staticmethod
    def block_definition():
        return GroupBlockSerializer.block_name, GroupBlock(stream_blocks=[
            InputBlockSerializer.block_definition()
        ])

    def to_representation(self, instance):
        return super(GroupBlockSerializer, self).to_representation(instance)

    # def get_api_representation(self, value, context=None):
    #     # we dont want to send action to frontend
    #     result = {a[0]: a[1] for a in value.items() if isinstance(a[1], str)}
    #     result['stream'] = self.stream_definition.get_api_representation(value['stream'])
    #     return result

class GroupBlock(blocks.StructBlock):

    def __init__(self, stream_blocks, **kwargs):
        self.stream_definition = blocks.StreamBlock(local_blocks=stream_blocks)
        local_blocks = [
            ('name', blocks.CharBlock(max_length=80)),
            ('row', blocks.BooleanBlock(required=False, default=False, label="Zobrazit na jednom řádku.")),
            ('required', blocks.ChoiceBlock(choices=choices, default="always", label="Podmínka zobrazení pole")),
            ('stream', self.stream_definition)
        ]
        super().__init__(local_blocks=local_blocks, **kwargs)




# Generated by Django 3.2 on 2021-05-03 17:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('generic_forms', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='formbuilder',
            name='content',
        ),
    ]



from django.db import models
from modelcluster.models import ClusterableModel
from wagtail.core.fields import StreamField
from wagtail.snippets.models import register_snippet
from wagtail_rest_pack.generic_forms.blocks.group_block import GroupBlockSerializer
from wagtail_rest_pack.generic_forms.blocks.submit_block import SubmitBlockSerializer
from wagtail_rest_pack.generic_forms.blocks.text_block import InputBlockSerializer

security_choices = [
    ('recaptcha_or_user', 'Recaptcha nebo přihlášený uživatel.'),
    ('authenticated_user_only', 'Přihlášený uživatel pouze.')
]


@register_snippet
class FormBuilder(ClusterableModel):
    name = models.CharField(max_length=60, validators=[], primary_key=True)
    display_name = models.TextField(max_length=100, default="")
    description = models.TextField(max_length=1000, blank=True)
    created_on = models.DateTimeField(auto_now_add=True)
    security = models.CharField(max_length=30, default="recaptcha_or_user", choices=security_choices)

    stream = StreamField(block_types=[
        InputBlockSerializer.block_definition(),
        GroupBlockSerializer.block_definition(),
        SubmitBlockSerializer.block_definition(),
    ])

    def __str__(self):
        return 'Formulář "{}"'.format(self.display_name)



from django.http import HttpResponseNotFound, HttpResponseBadRequest, HttpResponse
from rest_framework import serializers
from rest_framework.views import APIView
from rest_framework.permissions import AllowAny

from wagtail_rest_pack.generic_forms.models import FormBuilder
from .models import FormBuilder


class PostFormSerializer(serializers.Serializer):
    name = serializers.CharField(max_length=60)
    action = serializers.CharField(max_length=30)

    class Meta:
        fields = ['name', 'action', ]

    def create(self, validated_data):
        return super(PostFormSerializer, self).create(validated_data)


class PostFormView(APIView):
    permission_classes = [AllowAny]
    lookup_url_kwarg = 'name'
    queryset = FormBuilder.objects.all()

    def get_serializer(self, *args, **kwargs):
        return PostFormSerializer(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data, *args, **kwargs)
        if serializer.is_valid():
            data = serializer.validated_data
            form: FormBuilder = self.queryset.get(name=data['name'])
            serializer.save()
            return HttpResponse('cajk')
        return HttpResponseBadRequest(serializer.errors['name'])




from rest_framework import serializers,generics
from rest_framework.permissions import AllowAny
from wagtail_rest_pack.streamfield.serializers import SettingsStreamFieldSerializer

from .models import FormBuilder
from wagtail.core.fields import StreamField
from wagtail.api.v2.serializers import Field




class GetFormBuilderSerializer(serializers.ModelSerializer):
    stream = SettingsStreamFieldSerializer()

    class Meta:
        model = FormBuilder
        fields = ['name', 'display_name', 'security', 'stream']


class FormView(generics.RetrieveAPIView):

    permission_classes = [AllowAny]
    queryset = FormBuilder.objects.all()
    lookup_field = 'name'
    lookup_url_kwarg = 'name'
    serializer_class = GetFormBuilderSerializer
